import pyglet
import Box2D as box2d
from worldcontext import WorldContext
from crow import Murder, Crow
from crate import Crate
import physics
import random
import biped
import settings
import math
import gamestats

try:
    import psyco
    psyco.full()
except:
    pass

from weather import Weather



class ContactListener(box2d.b2ContactListener):
    
    def Add(self, point):
        e1 = point.shape1.GetBody().userData
        e2 = point.shape2.GetBody().userData
        if e1: e1.on_hit(point)
        if e2: e2.on_hit(point)
    
    def Remove(self, point):
        e1 = point.shape1.GetBody().userData
        e2 = point.shape2.GetBody().userData
        if e1: e1.on_remove_hit(point)
        if e2: e2.on_remove_hit(point)
    
    def Persist(self, point):
        e1 = point.shape1.GetBody().userData
        e2 = point.shape2.GetBody().userData
        if (isinstance(e1, Crow) and isinstance(e2, biped.Biped)) or (isinstance(e2, Crow) and isinstance(e1, biped.Biped)):
            if point.velocity.Length() < 10*settings.scale:
                e1.damage(point)
                e2.damage(point)

class WinContext(WorldContext):
    def __init__(self, window):
        WorldContext.__init__(self, "WinContext")
        self.window = window
        self.labels = []
        
    
    def on_activate(self):
        window = self.window
        self.labels.append( pyglet.text.Label("""You are made of win!""",
                            font_name="Tahoma",
                            font_size=12,
                            x=window.width//2, y=window.height//2,
                            anchor_x='center', anchor_y='center',
                            color=(255, 255, 255,255)))
        self.labels.append( pyglet.text.Label("""Lives Taken: """ + str(gamestats.lives_taken),
                            font_name="Tahoma",
                            font_size=12,
                            x=window.width//2, y=window.height//2-20,
                            anchor_x='center', anchor_y='center',
                            color=(255, 255, 255,255)))
        self.labels.append( pyglet.text.Label("""Limbs Consumed: """ + str(gamestats.limbs_taken),
                            font_name="Tahoma",
                            font_size=12,
                            x=window.width//2, y=window.height//2-40,
                            anchor_x='center', anchor_y='center',
                            color=(255, 255, 255,255)))
        
        self.labels.append( pyglet.text.Label("""Survivors: """ + str(gamestats.survivors),
                            font_name="Tahoma",
                            font_size=12,
                            x=window.width//2, y=window.height//2-60,
                            anchor_x='center', anchor_y='center',
                            color=(255, 255, 255,255)))
    
    def draw_ui(self):
        
        for x in self.labels:
            x.draw()
        
class MapContext(WorldContext):
    song = 'data/music/surfin_bird.mp3'
    
    def __init__(self, window, name):
        WorldContext.__init__(self, name)
        self.window = window
        self.mouse_down = False
        self.init_world()
        self.time = 0
        self.spawn_countdown = 6
        self.spawn_rate = 1
        self.spawn_time = 10
        self.average_speed = 2
    
    def on_activate(self):
        self.music = pyglet.resource.media(self.song)
        self.music = self.music.play()
    
    def init_world(self):
        self.init_physics()
        self.murder = Murder(self)
        for i in range(settings.crow_count):
            self.murder.add_crow(box2d.b2Vec2(random.random()*settings.world_width, random.randint(int(settings.world_height*0.8), int(settings.world_height))))
    
    def init_physics(self):
        # Box2D Initialization
        self.worldAABB = box2d.b2AABB()
        self.worldAABB.lowerBound = (-self.window.width, -self.window.height)
        self.worldAABB.upperBound = (self.window.width*2, self.window.height*2)
        gravity = (0.0, -20.0)
        
        doSleep = True
        self.contact_listener = ContactListener()
        self.world = box2d.b2World(self.worldAABB, gravity, doSleep)
        self.world.SetContactListener(self.contact_listener)
        self.build_map_container()
        self.build_ground()
        self.build_objects()
        
        self.moon = pyglet.sprite.Sprite(img=pyglet.resource.image('data/moon.png'), x=settings.world_width*.75, y=settings.world_height*.75 )
        self.moon.scale = settings.scale
        self.weather = Weather(self)
        self.scarecrow = pyglet.sprite.Sprite(pyglet.resource.image('data/scarecrow.png'), x=80, y=12)
        self.scarecrow.scale = settings.scale
        
    def build_map_container(self):
        bd=box2d.b2BodyDef() 
        sd=box2d.b2PolygonDef()
        bd.position = (settings.world_width//2, settings.world_height//2)
        self.container = self.world.CreateBody(bd)
        
        sd.SetAsBox(settings.world_width, 10.0*settings.scale, box2d.b2Vec2(0, -settings.world_height/2-5*settings.scale), 0 )
        self.container.CreateShape(sd)
        
        sd.SetAsBox(settings.world_width, 10.0*settings.scale, box2d.b2Vec2(0, settings.world_height/2+5*settings.scale), 0 )
        self.container.CreateShape(sd)
        
        sd.SetAsBox(10.0*settings.scale, settings.world_width, box2d.b2Vec2(settings.world_width/2+5*settings.scale, 0), 0 )
        self.container.CreateShape(sd)
        
        sd.SetAsBox(10.0*settings.scale, settings.world_width, box2d.b2Vec2(-settings.world_width/2-5*settings.scale, 0), 0 )
        self.container.CreateShape(sd)
    
    def build_ground(self):
        def make_polygon(vertices):
            return physics.Polygon(physics.PolygonDef(self.world,vertices, texture='data/ground.png'), (0,0), has_mass=False)
        self.ground = [
            make_polygon([(0,0, 0, 0), (30, 0, 1, 0), (30, 10, 1, 1), (20, 24, 0.5, 1), (0, 25, 0, 1)]),
            make_polygon([(30,0, 0, 0), (40, 0, 1, 0), (40, 8, 1, 1), (30, 10, 0, 1)]),
            make_polygon([(40,0, 0, 0), (60, 0, 1, 0), (60, 8, 1, 1), (40, 8, 0, 1)]),
            make_polygon([(60,0, 0, 0), (90, 0, 1, 0), (90, 15, 1, 1), (60, 8, 0, 1)]),
            make_polygon([(90,0, 0, 0), (100, 17, 1, 1), (90, 15, 0, 1)]),
            make_polygon([(100,0, 0, 0), (160, 0, 1, 0), (160, 5, 1, 1), (100, 5, 0, 1)]),
            make_polygon([(85,50, 0, 1), (90,45, 1, 0), (100, 40, 1.5, 0), (160, 40, 5, 0), (160, 50, 5, 1)])
        ]
        
    
    def build_objects(self):
        cd = physics.CircleDef(self.world, 3.0, density=0.2)
        self.things = [physics.Circle(cd, (60, 22)),
                       Crate(self.world, (100, 55)),
                       Crate(self.world, (100, 59)),
                       Crate(self.world, (95,  55)),
                       Crate(self.world, (110, 55))]
        self.people = []
    
    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.C:
            for person in self.people:
                person.remove_from_world()
            self.people = []
        if symbol == pyglet.window.key.W:
            self.switch_requested("WinContext")
    
    def on_mouse_press(self, x, y, button, modifiers):
        self.mouse_down = True
        if button == 1:
            speed = settings.crow_fast_speed
        else:
            speed = settings.crow_slow_speed
            
        for crow in self.murder.crows:
            crow.resting = False
            crow.speed = speed
            crow.moveTowards(random.random() * 3 + x, random.random() * 3 + y )
        
    def on_mouse_release(self, x, y, button, modifiers):
        if button == 4:
            for crow in self.murder.crows:
                crow.target = None
                crow.speed = settings.crow_fast_speed
                #crow.moveTowards(random.random() * 12 + x, random.random() * 12 + y )
        if button == 1:
            for crow in self.murder.crows:
                #crow.moveTowards(random.random() * 12 + x, random.random() * 12 + y )
                crow.resting = False
                crow.target = box2d.b2Vec2(random.randint(0, int(settings.world_width)),
                                           random.randint(int(settings.world_height*0.8), int(settings.world_height)))
                crow.speed = settings.crow_idle_speed
            #crow.moveTowards(random.random() * 6 + x, random.random() * 6 + y )
                
            #
        self.mouse_down = False
    
    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        if button == 1:
            for crow in self.murder.crows:
                crow.moveTowards(random.random() + x, random.random() + y )
        else:
            for crow in self.murder.crows:
                crow.moveTowards(x, y )
    
    @property
    def width(self):
        return self.worldAABB.upperBound.x
    
    @property
    def height(self):
        return self.worldAABB.upperBound.y
    
    def spawn_enemies(self, dt):
        self.spawn_countdown -= dt
        if self.spawn_countdown <= 0:
            self.spawn_countdown = self.spawn_time
            if self.spawn_countdown < 2.5: self.spawn_countdown = 2.5
            self.average_speed += 0.2
            self.spawn_time -= 0.4
            if self.spawn_time < 2: self.spawn_countdown = 2
            for x in range(int(self.spawn_rate)):
                self.people.append(biped.Biped(self.world, (x*2, 30), self.average_speed + random.random()))
            self.spawn_rate += 0.2
            print self.spawn_countdown, self.spawn_rate, self.average_speed
    
    def update(self, dt):
        if self.time >= 140:
            self.switch_requested("WinContext")
        
        self.time += dt
        self.spawn_enemies(dt)
        for thing in self.things:
            thing.update(dt)
        for person in self.people:
            person.update(dt)
            if len(self.people) > 10 and len(person.joints) <= 4:
                person.remove_from_world()
            elif person.Chest.position.x >= settings.world_width - 2 and not person.is_dead:
                person.remove_from_world()
                gamestats.add_survivor()
        if len(self.people) > 8:
            self.people = [x for x in self.people if len(x.joints) > 0]
        
        self.murder.update(dt)
        self.weather.update(dt)
        self.world.SetContinuousPhysics(False)

        
        # Tell Box2D to step
        if dt > .1: dt = .1
        self.world.Step(dt, 10, 10)
        self.world.Validate()
            
    def draw(self):
        self.moon.draw()
        self.weather.draw_background()
        self.scarecrow.draw()
        
        
        
        pyglet.gl.gl.glColor4f(0,0,0,1)
        for ground in self.ground:
            ground.draw()
        self.murder.draw()
        for thing in self.things:
            thing.draw()
        for person in self.people:
            person.draw()
        self.weather.draw_foreground()
        